import mesa
from src.Behavior import Citizen


class Agent(mesa.Agent):
    """An agent with fixed initial wealth."""

    def __init__(self, unique_id, model):
        # Pass the parameters to the parent class.
        super().__init__(unique_id, model)
        self.next_steps = []
        self.target = None
        self.isMoving = False
        self.task = "home"
        self.behavior = Citizen(self.model, self)

        self.model.grid.place_agent(self, self.behavior.home)


    def step(self):
        self.move()

    def move(self):
        if len(self.next_steps) > 0:
            next_step = self.next_steps.pop(0)
            self.model.grid.move_agent(self, next_step)

            # check if length is greater than one to prevent agents save there position if standing
            if self.isMoving:
                # remember step in record based on the size in behavior (citizen size is 1)
                self.model.nav.save_step_to_record(next_step[0], next_step[1], self.behavior.size_on_record)

        else:
            self.target = self.behavior.get_new_target()
            self.next_steps = self.model.nav.calc_path(self.pos, self.target)

            # get current agent state
            self.task = self.behavior.get_task()


        # check if walking
        self.isMoving = self.pos[0] != self.target[0] or self.pos[1] != self.target[1]
