import random
from src.helper import *
from config import *


class Citizen:
    def __init__(self, model, agent):
        self.model = model
        self.agent = agent
        self.nav = model.nav

        # defines the impact/amount for each step in the record
        self.size_on_record = 1

        self.day_plan_from = -1

        self.day_plan = ["home"] * 24
        self.daily_work = random.randint(8, 14)

        self.works_in_city = chance(WORKS_IN_CITY_RATE)
        self.lives_in_city = chance(LIVES_IN_CITY_RATE)
        self.works_on_sunday = chance(WORK_ON_SUNDAY_RATE)
        self.is_christian = chance(CHRISTIAN_RATE)

        if self.lives_in_city:
            self.home = self.nav.random_street_pixel()
        else:
            self.home = self.nav.random_outgoing_connection_pixel()

        if self.works_in_city:
            self.working_place = self.nav.random_street_pixel()
        else:
            self.working_place = self.nav.random_outgoing_connection_pixel()

        self.church = self.nav.nearest_church(self.home)

    def clear_day(self):
        for hour in range(0, 24):
            self.day_plan[hour] = "home"

    def generate_day(self, current_day):
        if IS_SUNDAY:
            self.generate_sunday()
        else:
            self.generate_work_day()

    def generate_sunday(self):
        self.clear_day()

        # create base day
        for hour in range(6, 22):
            going_home = chance(0.5)
            self.day_plan[hour] = "home" if going_home else "random_target"

        # going to church (9-11)
        if self.is_christian:
            self.day_plan[9] = self.day_plan[11] = self.day_plan[10] = "church"

        # calc if the agents works or goes to the market
        if self.works_on_sunday:
            for i in range(12, 17):
                self.day_plan[i] = "work"
        elif chance(MARKT_VISIT_RATE):
            visit_time = random.randint(12, 17)
            self.day_plan[visit_time] = "market"

    def generate_work_day(self):
        self.clear_day()

        start_work = random.randint(4, 6)
        remaining_work_hour = self.daily_work
        remaining_work_blocks = 3
        hour_pointer = start_work

        # works in 1 to 3 block of 4 to 6
        # 1 to 2 hours break between block
        # 8 to 14 hours per day
        while remaining_work_hour > 0 and remaining_work_blocks > 0:
            # calc work block
            remaining_work_blocks -= 1
            block_duration = min(random.randint(4, 6), remaining_work_hour)
            remaining_work_hour -= block_duration

            # add work block
            for i in range(0, block_duration):
                self.day_plan[hour_pointer] = "work"
                hour_pointer += 1

            if hour_pointer > 22:
                break

            # calc free time
            going_home = chance(0.5)
            free_time_duration = random.randint(1, 2)

            for i in range(0, free_time_duration):
                if going_home:
                    self.day_plan[hour_pointer] = "home"
                else:
                    self.day_plan[hour_pointer] = "random_target"
                hour_pointer += 1

            if hour_pointer > 22:
                break

    def get_new_target(self):
        current_step = self.model.get_step_count()
        current_day = tick_to_day(current_step)

        # generate a new day if day is over
        if self.day_plan_from != current_day:
            self.generate_day(current_day)
            self.day_plan_from = current_day

        current_task = self.get_task()

        if current_task == "home":
            return self.home
        if current_task == "work":
            return self.working_place
        if current_task == "random_target":
            return self.nav.random_street_pixel()
        if current_task == "church":
            return self.church
        if current_task == "market":
            return self.nav.random_market_pixel()

    # returns the current task of the agent
    def get_task(self):
        current_step = self.model.get_step_count()
        current_hour = tick_to_hour(current_step)

        return self.day_plan[current_hour]
