from mesa.visualization.modules import CanvasGrid, ChartModule
from mesa.visualization.ModularVisualization import ModularServer
from mesa.visualization.UserParam import Slider

from src.Model import Model

NUMBER_OF_CELLS = 256

SIZE_OF_CANVAS_IN_PIXEL_W = 1024
SIZE_OF_CANVAS_IN_PIXEL_H = 1024

simulation_params = {
    "number_of_agents": Slider(
        "Number of agents",
        value=500,
        min_value=1,
        max_value=500,
        step=1,
        description="Number of agents",
    ),

    "weight_terrain_map": Slider(
        "Weight of terrain map cost",
        value=1,
        min_value=1,
        max_value=5,
        step=0.1,
        description="",
    ),

    "weight_base_cost": Slider(
        "Weight of base cost",
        value=1,
        min_value=1,
        max_value=5,
        step=1,
        description="",
    ),

    "weight_record_cost": Slider(
        "Weight of base cost",
        value=1,
        min_value=1,
        max_value=5,
        step=0.1,
        description="",
    ),

    "safe_to_disk": False,
}


def agent_portrayal(agent):
    radius = 6 if agent.isMoving else 4

    color = "black"
    if agent.task == "home":
        color = "green"
    if agent.task == "work":
        color = "red"
    if agent.task == "random_target":
        color = "blue"
    if agent.task == "market":
        color = "yellow"
    if agent.task == "church":
        color = "orange"

    portrayal = {"Shape": "circle", "r": radius, "Filled": "true", "Layer": 1, "Color": color}

    return portrayal


grid = CanvasGrid(
    agent_portrayal,
    NUMBER_OF_CELLS,
    NUMBER_OF_CELLS,
    SIZE_OF_CANVAS_IN_PIXEL_W,
    SIZE_OF_CANVAS_IN_PIXEL_H
)

chart_currents = ChartModule(
    [
        {"Label": "Moving Agents", "Color": "black"}
    ],
    canvas_height=300,
    data_collector_name="datacollector_moving"
)

chart_tasks = ChartModule(
    [
        {"Label": "Home", "Color": "green"},
        {"Label": "Work", "Color": "red"},
        {"Label": "Random", "Color": "blue"},
        {"Label": "Church", "Color": "orange"},
        {"Label": "Market", "Color": "yellow"},
    ],
    canvas_height=300,
    data_collector_name="datacollector_task"
)

server = ModularServer(Model, [grid, chart_currents, chart_tasks], "City Model", simulation_params)
server.port = 8515
